@@ -5,16 +5,6 @@ use Mojo::Util qw(hmac_sha1_sum);
 use Hash::Merge qw( merge );
 use DBIx::POS::Sth;
 
-our $VERSION = '0.701';
-
-=pod
-
-=head1 VERSION
-
-0.701
-
-=cut
-
 my $pkg = __PACKAGE__;
 
 has [qw(app dbh conf)];
@@ -330,7 +320,7 @@ sub deny_log {
   );
 }
 
-1;
+our $VERSION = '0.705';
 
 =pod
 
@@ -342,6 +332,10 @@ sub deny_log {
 
 ¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
 
+=head1 VERSION
+
+0.705
+
 =head1 NAME
 
 Mojolicious::Plugin::RoutesAuthDBI - from DBI sql-tables does generate routes, make authentication and make restrict access (authorization) to request. Plugin makes an auth operations throught the plugin L<Mojolicious::Plugin::Authentication> on which is based.
@@ -411,7 +405,7 @@ See L<Mojolicious::Plugin::RoutesAuthDBI::Access> for detail options list.
 Hashref options for admin controller for actions on SQL tables routes, roles, profiles, logins. By default the builtin module:
 
   admin => {
-    controller => 'Access',
+    controller => 'Admin',
     namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
     ...,
   },
