@@ -1,10 +1,10 @@
 package Mojolicious::Plugin::RoutesAuthDBI::Admin;
 use Mojo::Base 'Mojolicious::Controller';
-#~ use Mojolicious::Plugin::RoutesAuthDBI::Sth;#  sth cache
+
 
 my $pkg = __PACKAGE__;
 my ($dbh, $sth, $init_conf);
-has [qw(dbh sth)];
+has [qw(dbh sth plugin)];
 
 sub init {# from plugin! init Class vars
   my $self = shift;
@@ -17,6 +17,7 @@ sub init {# from plugin! init Class vars
   $sth = $self->sth
     or die "Нет STH";
   #~ $app = $self->app($self->{app} || $args{app});
+  $self->plugin($self->{plugin} || $args{plugin});
   $init_conf = $self;
   return $self;
 }
@@ -32,7 +33,7 @@ You are signed as:
 @{[$c->dumper( $c->auth_user)]}
 
 
-ADMIN ROUTES
+ADMIN ROUTES (this controller)
 ===
 
 @{[map "$_->{request}\t\t$_->{descr}\n", $c->self_routes]}
@@ -65,17 +66,31 @@ sub signout {
   
 }
 
+sub users {
+  my $c = shift;
+  
+  my $p = $dbh->selectall_arrayref($sth->sth('profiles'), {Slice=>{}},);
+  $c->render(format=>'txt', text=><<TXT)
+$pkg
+
+Profiles
+===
+
+@{[$c->dumper( $p)]}
+TXT
+}
+
 sub new_user {
   my $c = shift;
   
   my ($login, $pass) = $c->vars('login', 'pass');
   
   my $r;
-  ($r = $dbh->selectrow_hashref($sth->sth('user'), undef, (undef, $login)))
+  ($r = $dbh->selectrow_hashref($sth->sth('profile'), undef, (undef, $login)))
     and $c->render(format=>'txt', text=><<TXT)
 $pkg
 
-User already exists
+Profile/User already exists
 ===
 
 @{[$c->dumper( $r)]}
@@ -83,17 +98,23 @@ TXT
     and ($r->{not_new} = '!')
     and return $r;
   
-  $r = $dbh->selectrow_hashref($sth->sth('new user'), undef, ($login, $pass));
+  $r = $dbh->selectrow_hashref($sth->sth('new login'), undef, ($login, $pass));
+  
+  my $p =  $dbh->selectrow_hashref($sth->sth('new profile'), undef, ([$login],));
+  
+  $c->ref($p->{id} => $r->{id});
+  
+  @$p{qw(login pass)} = @$r{qw(login pass)};
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-Success sign up new user
+Success sign up new profile/user
 ===
 
-@{[$c->dumper( $r)]}
+@{[$c->dumper( $p)]}
 TXT
-  return $r;
+  return $p;
 }
 
 sub trust_new_user {
@@ -106,7 +127,7 @@ sub trust_new_user {
   $rl ||= $dbh->selectrow_hashref($sth->sth('new role'), undef, ('admin'));
   
   # REF role->user
-  my $ru = $c->ref($rl->{id}, $u->{id});
+  my $ru = $c->ref($rl->{id} => $u->{id});
   
   # CONTROLLER
   my $cc = $dbh->selectrow_hashref($sth->sth('controller', where=>"where controller=? and (namespace=? or (?::varchar is null and namespace is null))"), undef, ($init_conf->{controller}, ($init_conf->{namespace}) x 2,));
@@ -174,31 +195,33 @@ TXT
 
 sub user_roles {
   my $c = shift;
-  my ($user) = $c->vars('user');
-  my $u =  $dbh->selectrow_hashref($sth->sth('user'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
+  my ($user) = $c->vars('user') || $c->vars('login');
+  my $u =  $dbh->selectrow_hashref($sth->sth('profile'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
-No such user [$user]
+No such profile.id/login [$user]
 ===
 
 TXT
     and return
     unless $u;
   
-  my $r = $dbh->selectall_arrayref($sth->sth('user roles'), { Slice => {} }, ($u->{id}));
+  my $r = $dbh->selectall_arrayref($sth->sth('profile roles'), { Slice => {} }, ($u->{id}));
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-List of user roles (@{[scalar @$r]})
-===
-
-USER
+PROFILE+LOGIN
+---
 @{[$c->dumper( $u)]}
 
+List of profile/login roles (@{[scalar @$r]})
+===
+
 ROLES
+---
 @{[$c->dumper( $r)]}
 
 TXT
@@ -222,24 +245,24 @@ TXT
     unless $r && $role =~ /\w/;
   $r ||= $dbh->selectrow_hashref($sth->sth('new role'), undef, ($role)) ;
   
-  my $user = $c->stash('user') || $c->param('user');
-  my $u =  $dbh->selectrow_hashref($sth->sth('user'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
+  my ($user) = $c->vars('user');
+  my $u =  $dbh->selectrow_hashref($sth->sth('profile'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
-No such user [$user]
+No such profile.id/login [$user]
 ===
 
 TXT
     and return
     unless $u;
   
-  my $ref = $dbh->selectrow_hashref($sth->sth('ref'), undef, ($r->{id}, $u->{id}));
+  my $ref = $c->refs($r->{id} => $u->{id});
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
-Allready ref ROLE -> USER
+Allready ref ROLE[$r->{name}] -> USER [@{[$c->dumper( $u) =~ s/\s+//gr]}]
 ===
 
 @{[$c->dumper( $ref)]}
@@ -247,12 +270,12 @@ TXT
     and return
     if $ref;
   
-  $ref = $c->ref($r->{id}, $u->{id});
+  $ref = $c->ref($r->{id} => $u->{id});
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-Success create ref ROLE -> USER
+Success create ref ROLE[$r->{name}] -> USER [@{[$c->dumper( $u) =~ s/\s+//gr]}]
 ===
 
 @{[$c->dumper( $ref)]}
@@ -277,13 +300,13 @@ TXT
     and return
     unless $r;
 
-  my $user = $c->stash('user') || $c->param('user');
-  my $u =  $dbh->selectrow_hashref($sth->sth('user'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
+  my ($user) = $c->vars('user');
+  my $u =  $dbh->selectrow_hashref($sth->sth('profile'), undef, ($user =~ /\D/ ? (undef, $user) : ($user, undef,)));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
-No such user [$user]
+No such profile.id/login [$user]
 ===
 
 TXT
@@ -294,7 +317,7 @@ TXT
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
-Success delete ref ROLE[$role] -> USER[$user]
+Success delete ref ROLE[$r->{name}] -> USER[@{[$c->dumper( $u) =~ s/\s+//gr]}]
 ===
 
 @{[$c->dumper( $ref)]}
@@ -305,7 +328,7 @@ TXT
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
-There is no ref ROLE[$role] -> USER[$user]
+There is no ref ROLE[$r->{name}] -> USER[@{[$c->dumper( $u) =~ s/\s+//gr]}]
 
 TXT
   
@@ -359,11 +382,11 @@ TXT
     and return
     unless $r;
   
-  my $u = $dbh->selectall_arrayref($sth->sth('role users'), { Slice => {} }, ($r->{id}));
+  my $u = $dbh->selectall_arrayref($sth->sth('role profiles'), { Slice => {} }, ($r->{id}));
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-All @{[scalar @$u]} users by role [$r->{name}]
+All @{[scalar @$u]} profile/users by role [$r->{name}]
 ===
 
 @{[$c->dumper( $u)]}
@@ -655,6 +678,7 @@ TXT
   my $list2 = $act->{id} ? $dbh->selectall_arrayref($sth->sth('action routes', where=>'where action_id is null'), { Slice => {} }, ())
     : [];
   
+  no warnings;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -752,8 +776,8 @@ sub self_routes {# from plugin!
 /$prefix/role/dsbl/:role	disable_role	$prefix disable role->user	1	Disable :role by role.id|role.name.
 /$prefix/role/enbl/:role	enable_role	$prefix enable role->user	1	Enable :role by role.id|role.name.
 /$prefix/roles	roles	$prefix view roles	1	View roles table
-/$prefix/roles/:user	user_roles	$prefix roles of user	1	View roles of :user by id|login
-/$prefix/role/:role/:user	new_role_user	$prefix create ref role->user	1	Assign :user to :role by user.id|user.login and role.id|role.name.
+/$prefix/roles/:user	user_roles	$prefix roles of user	1	View roles of :user by profile.id|login
+/$prefix/role/:role/:user	new_role_user	$prefix create ref role->profile	1	Assign :user to :role by profile.id|login and role.id|role.name.
 #
 # Последовательный ввод нового маршрута
 #
@@ -769,17 +793,17 @@ sub self_routes {# from plugin!
 /$prefix/routes/:role	role_routes	$prefix routes of role	1	All routes of :role by id|name
 /$prefix/route/:route/:role	ref	$prefix create ref route->role	1	Assign :route with :role by route.id and role.id|role.name
 #
-# Пользователи
+# Пользователи/профили
 #
 /$prefix/user/new	new_user	$prefix create user	1	Add new user by params: login,pass,...
 /$prefix/user/new/:login/:pass	new_user	$prefix create user st	1	Add new user by :login & :pass
 /$prefix/users	users	$prefix view users	1	View users table
 /$prefix/users/:role	role_users	$prefix users of role	1	View users of :role by id|name
 #
-get foo /sign/in	sign	signin form	0	Login&pass form
-post /sign/in	sign	signin params	0	Auth by params
-/sign/in/:login/:pass	sign	signin stash	0	Auth by stash
-/sign/out	signout	go away	1	Exit
+#get foo /sign/in	sign	signin form	0	Login&pass form
+#post /sign/in	sign	signin params	0	Auth by params
+/$prefix/sign/in/:login/:pass	sign	signin stash	0	Auth by stash
+/$prefix/sign/out	signout	go away	1	Exit
 #
 /$prefix/$trust/admin/new/:login/:pass	trust_new_user	$prefix/$trust !trust create user!	0	Add new user by :login & :pass and auto assign to role 'Admin' and assign to access this controller!
 
@@ -841,23 +865,30 @@ Both above options determining the module controller for web actions on tables r
 
 =item * B<trust> is a url subprefix for trust admin urls of this module. See defaults below.
 
+=item * B<pos> - hashref
+
+SQL-dictionary for DBI statements. See L<Mojolicious::Plugin::RoutesAuthDBI::POS::Admin>.
+
 =back
 
 =head2 Defaults
 
-    admin = > {
-        namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
-        module => 'Admin',
-        prefix => 'admin', # lc(<module>)
-        trust => $app->secrets->[0],
-        pos => {...}, # sets from plugin options
-    },
+  admin = > {
+    namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
+    module => 'Admin',
+    prefix => 'admin', # lc(<module>)
+    trust => hmac_sha1_sum('admin', $app->secrets->[0]),
+    pos => {
+      namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
+      module => 'POS::Admin',
+    }, 
+  },
     
-    admin = {}, # empty hashref sets defaults above
-    
-    admin => undef, # disables routing of admin controller
-    
-    admin = > {prefix=>'myadmin', trust => 'foooobaaar'},# admin urls like: /myadmin/foooobaaar/.....
+  admin = {}, # empty hashref sets defaults above
+  
+  admin => undef, # disable admin controller
+  
+  admin = > {prefix=>'myadmin', trust => 'foooobaaar'},# admin urls like: /myadmin/foooobaaar/
 
 
 
