@@ -4,46 +4,71 @@ use Mojolicious::Plugin::RoutesAuthDBI::Util qw(json_enc json_dec);
 use Hash::Merge qw( merge );
 use Digest::MD5 qw(md5_hex);
 
-my ($dbh, $sth, $init_conf);
+my ($dbh, $sth, $Init);
 has [qw(app dbh sth plugin)];
 
 has _providers => sub {# default
   {
     vkontakte => {
-      key           => "0.........0",
-      secret        => "z.................8",
       authorize_url => "https://oauth.vk.com/authorize",
       authorize_query => {display=>'page', response_type=>'code', v=>'5.52',},#&scope=friends
       token_url     => "https://oauth.vk.com/access_token",
       profile_url => 'https://api.vk.com/method/users.get',#?user_ids=260362925&v=5.52&access_token=...
+      profile_query => sub {
+        my ($c, $auth, ) = @_;
+        {
+          access_token=>$auth->{access_token},
+          fields=>'photo_100',
+        };
+      },
     },
     google=>{# обязательно redirect_url
-      key=>'9................0.apps.googleusercontent.com',
-      secret=>'J...............1',
       scope=>'profile',
       profile_url=> 'https://www.googleapis.com/oauth2/v1/userinfo',
+      profile_query => sub {
+        my ($c, $auth, ) = @_;
+        {
+          alt => 'json',
+          access_token => $auth->{access_token},
+        };
+      },
     },
     yandex=>{# обязательно redirect_url
-      key=>'9.............d',
-      secret=>'5................f',
       authorize_url=>"https://oauth.yandex.ru/authorize",
       authorize_query => {force_confirm=>1, response_type=>'code',},# state=>
       token_url => "https://oauth.yandex.ru/token",
       profile_url=> "https://login.yandex.ru/info",
+      profile_query => sub {
+        my ($c, $auth, ) = @_;
+        {
+          format => 'json',
+          oauth_token=> $auth->{access_token},
+        };
+      },
     },
     mailru => {
-      key=>'z..........q',
-      secret => '1...............9',
       authorize_url=>"https://connect.mail.ru/oauth/authorize?response_type=code",
       token_url => "https://connect.mail.ru/oauth/token",
-      profile_url=> "https://www.appsmail.ru/platform/api"
+      profile_url=> "https://www.appsmail.ru/platform/api",
+      profile_query => sub {
+        my ($c, $auth, ) = @_;
+        my $param = {
+          method=>'users.getInfo',
+          app_id=>$Init->config->{mailru}{key},
+          session_key=>$auth->{access_token},
+          #~ uids=>$auth->{x_mailru_vid},
+          secure=>1,
+        };
+        $param->{sig} = md5_hex map("$_=$param->{$_}", sort keys %$param), $Init->config->{mailru}{secret};
+        $param;
+      },
 
     }
   }
   
 };
 
-has config => sub {# только $init_conf !
+has config => sub {# только $Init !
   my $self = shift;
   
   while (my ($name, $val) = each %{$self->{providers}}) {
@@ -55,39 +80,6 @@ has config => sub {# только $init_conf !
   merge $self->{providers}, $self->_providers;
 };
 
-has profile_urls => sub { {
-  vkontakte => sub {
-    my ($c, $profile_url, $auth, ) = @_;
-    $profile_url
-      ->query(sprintf qq{user_ids=%s&access_token=%s}, @$auth{qw(user_id access_token)}, );
-      #&v=%s     $c->site->{authorize_query}{v}
-  },
-  google => sub {
-    my ($c, $profile_url, $auth, ) = @_;
-    $profile_url
-      ->query('alt=json&access_token='.$auth->{access_token});
-  },
-  yandex => sub {
-    my ($c, $profile_url, $auth, ) = @_;
-    $profile_url
-      ->query('format=json&oauth_token='.$auth->{access_token});
-  },
-  mailru => sub {
-    my ($c, $profile_url, $auth, ) = @_;
-    #~ ?&&&sig=f82efdd230e45e58e4fa327fdf92135d&uids=15410773191172635989
-    my $param = {
-      method=>'users.getInfo',
-      app_id=>$init_conf->config->{mailru}{key},
-      session_key=>$auth->{access_token},
-      #~ uids=>$auth->{x_mailru_vid},
-      secure=>1,
-    };
-    $param->{sig} = md5_hex map("$_=$param->{$_}", sort keys %$param), $init_conf->config->{mailru}{secret};
-    $profile_url
-      ->query($param);
-  },
-}};
-
 has ua => sub {shift->app->ua->connect_timeout(30);};
 
 sub init {# from plugin
@@ -109,7 +101,7 @@ sub init {# from plugin
   #~ $self->app->log->debug($self->app->dumper($self->config));
   $self->app->plugin("OAuth2" => $self->config);
   
-  $init_conf = $self;
+  $Init = $self;
   return $self;
   
 }
@@ -117,17 +109,29 @@ sub init {# from plugin
 sub login {
   my $c = shift;
   
-  my $redirect = $c->param('redirect') || 'profile';
+  $c->session(oauth_init => {
+    redirect => $c->param('redirect') || ($c->req->headers->referrer && Mojo::URL->new($c->req->headers->referrer)->path) || 'profile',
+    #~ $c->param('fail_render') ? (fail_render => $c->param('fail_render')) : (),
+  })
+    unless $c->session('oauth_init');
   
   my $site_name = $c->stash('site');
 
   my $site = $c->oauth2->providers->{$site_name}
     or die "No such oauth provider [$site_name]" ;
   
-  die "OAuth provider [$site_name] does not configured"
-    unless $site->{id};
-    
-  my $fail_auth_cb = $init_conf->{fail_auth_cb};
+  if (my @fatal = grep !defined $site->{$_}, qw(id key secret authorize_url token_url profile_url profile_query)) {
+    die "OAuth provider [$site_name] does not configured: [@fatal] is not defined";
+  }
+  
+  #~ my $fail_auth_cb = $Init->{fail_auth_cb};
+  
+  my $auth_profile = $c->${ \$Init->plugin->merge_conf->{auth}{current_user_fn} };
+  
+  my $r; $r = $dbh->selectrow_hashref($sth->sth('check profile oauth'), undef, ($auth_profile->{id}, $site->{id}))
+    and $c->app->log->warn("Попытка двойной авторизации сайта $site_name", $c->dumper($r), "профиль: ", $c->dumper($auth_profile),)
+    and return $c->redirect_to($c->url_for(${ delete $c->session->{oauth_init} }{redirect})->query(err=> "Уже есть авторизация сайта $site_name"))
+    if $auth_profile;
 
   $c->delay(
     sub { # шаг авторизации
@@ -144,12 +148,11 @@ sub login {
         if $auth->{error};
       
       $c->app->log->error("Автоизация $site_name:", $err, $c->dumper($auth))
-        and return $c->$fail_auth_cb($err.' Нет access_token')
+        #~ and return $c->$fail_auth_cb()
+        and return $c->redirect_to($c->url_for(${ delete $c->session->{oauth_init} }{redirect})->query(err=> $err.' Нет access_token'))
         unless $auth->{access_token};
       
-      my $url = $c->${ \$c->profile_urls->{$site_name} }(Mojo::URL->new($site->{profile_url}), $auth)
-        or $c->app->log->error("Нет ссылки для профиля $site_name")
-        and return $c->$fail_auth_cb("Нет ссылки для профиля $site_name");
+      my $url = Mojo::URL->new($site->{profile_url})->query($c->${ \$site->{profile_query} }($auth));
       
       $c->ua->get($url, $delay->begin);
       $delay->pass($auth);
@@ -161,7 +164,8 @@ sub login {
         if ref($profile) eq 'HASH' && $profile->{error};
       
       $c->app->log->error("Профиль $site_name:", $err, $tx->req->url, $c->dumper($tx->res), $c->dumper($profile), )
-        and return $c->$fail_auth_cb($err)
+        #~ and return $c->$fail_auth_cb($err)
+        and return $c->redirect_to($c->url_for(${ delete $c->session->{oauth_init} }{redirect})->query(err=> $err))
         if $err;
         
       $profile = $profile->{response}
@@ -175,31 +179,26 @@ sub login {
       || $dbh->selectrow_hashref($sth->sth('new oauth user'), undef, @bind);
 
       #~ $c->app->log->debug("Oauth user row: ", $c->dumper($u));
-      my $auth_user_helper = $init_conf->plugin->merge_conf->{auth}{current_user_fn};
-      
-      my $current_auth = $c->$auth_user_helper;
       
       my $профиль = 
       
-        $current_auth
+        $auth_profile
         
         || $dbh->selectrow_hashref($sth->sth('profile by oauth user'), undef, ($u->{id}))
 
 
-        || $dbh->selectrow_hashref($init_conf->plugin->admin->sth->sth('new profile'), undef, ([$profile->{first_name} || $profile->{given_name}, $profile->{last_name} || $profile->{family_name},]));
+        || $dbh->selectrow_hashref($Init->plugin->admin->sth->sth('new profile'), undef, ([$profile->{first_name} || $profile->{given_name}, $profile->{last_name} || $profile->{family_name},]));
 
-      my $r = $init_conf->plugin->admin->ref($профиль->{id}, $u->{id},);
+      my $r = $Init->plugin->admin->ref($профиль->{id}, $u->{id},);
       
       $c->authenticate(undef, undef, $профиль)
-        unless $current_auth;
+        unless $auth_profile;
 
 
       #~ $c->app->log->debug("Профиль: ", $c->dumper($профиль));
-      #~ return $c->session(token => $c->redirect_to('profile'));
-      return $c->redirect_to($redirect);
+      return $c->redirect_to(${ delete $c->session->{oauth_init} }{redirect});
     },
   );
-  #~ $c->app->log->debug("Login delay done");
   
 }
 
@@ -207,7 +206,7 @@ sub login {
 sub out {# выход
   my $c = shift;
   $c->logout;
-  $c->redirect_to($c->param('redirect') || 'home');
+  $c->redirect_to($c->param('redirect') || '/');
 }
 
 sub _routes {# from plugin!
@@ -216,20 +215,20 @@ sub _routes {# from plugin!
   return (
   
   {request=>'/login/:site',
-    namespace=>$init_conf->{namespace},
-    controller=>$init_conf->{controller} || $init_conf->{module},
+    namespace=>$Init->{namespace},
+    controller=>$Init->{controller} || $Init->{module},
     action => 'login',
     name => 'oauth-login',
   },
   {request =>'/logout',
-    namespace=>$init_conf->{namespace},
-    controller=>$init_conf->{controller} || $init_conf->{module},
+    namespace=>$Init->{namespace},
+    controller=>$Init->{controller} || $Init->{module},
     action => 'out',
     name => 'logout',
   },
-  {request =>'/'.$init_conf->plugin->admin->{trust}."/oauth/conf",
-    namespace=>$init_conf->{namespace},
-    controller=>$init_conf->{controller} || $init_conf->{module},
+  {request =>'/'.$Init->plugin->admin->{trust}."/oauth/conf",
+    namespace=>$Init->{namespace},
+    controller=>$Init->{controller} || $Init->{module},
     action => 'conf',
     name => 'oauth-conf',
   }
@@ -284,15 +283,35 @@ See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagr
 
   providers => {google=>{key=> ..., secret=>..., }, ...},
 
+See L<Mojolicious::Plugin::OAuth2>. But two additional parameters (keys of provider hash) are needs:
+
+=over 4
+
+=item * B<profile_url> abs url string
+
+  profile_url=> 'https://www.googleapis.com/oauth2/v1/userinfo',
+
+=item * B<profile_query> coderef
+
+Example for google:
+
+  profile_query => sub {
+    my ($c, $auth, ) = @_;
+    {
+      alt => 'json',
+      access_token => $auth->{access_token},
+    };
+  },
+
+In: $auth hash ref with access_token.
+Out: hashref profile query params.
+
+=back
 
 =item * B<pos> - hashref
 
 SQL-dictionary for DBI statements. See L<Mojolicious::Plugin::RoutesAuthDBI::POS::OAuth2>.
 
-=item * B<fail_auth_cb> - coderef
-
-Invokes on diffrent api errors
-
 =back
 
 =head2 Defaults
@@ -304,19 +323,32 @@ Invokes on diffrent api errors
       namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
       module => 'POS::OAuth2',
     },
-    fail_auth_cb => sub {shift->render(format=>'txt', text=>"@_")},
   },
+
+disable oauth module
   
-  oauth => undef, # disable oauth
+  oauth => undef, 
   
 
 =head1 METHODS NEEDS IN PLUGIN
 
-=over 4
+=head2 _routes()
 
-=item * B<_routes()> - this oauth controller routes. Return array of hashrefs routes records for apply route on app.
+This oauth controller routes. Return array of hashrefs routes records for apply route on app. Plugin internal use.
 
-=back
+=head1 Routes
+
+=head2 /login/:site
+
+Main route of this controller. Stash B<site> is the name of the hash key of the C<providers> config above. Example html link:
+
+  <a href="<%= $c->url_for('oauth-login', site=> 'google')->query(redirect=>'profile') %>">Login by google</a>
+
+This route has name 'oauth-login'. This route accept param 'redirect' and will use for $c->redirect_to after success oauith and also failed oauth with out param 'err'.
+
+=head2 /logout
+
+Clear session and redirect to param 'redirect' || '/'. This route has name 'logout'.
 
 =head1 SEE ALSO
 
